import{d3 as W}from"./index-Cq8aT7Ma.js";const $=["abort","canplay","canplaythrough","durationchange","emptied","encrypted","ended","error","loadeddata","loadedmetadata","loadstart","pause","play","playing","progress","ratechange","seeked","seeking","stalled","suspend","timeupdate","volumechange","waiting","waitingforkey","resize","enterpictureinpicture","leavepictureinpicture","webkitbeginfullscreen","webkitendfullscreen","webkitpresentationmodechanged"],A=globalThis.document?.createElement("template");A&&(A.innerHTML=`
    <style>
      :host {
        display: inline-block;
        line-height: 0;
      }

      video,
      audio {
        max-width: 100%;
        max-height: 100%;
        min-width: 100%;
        min-height: 100%;
      }
    </style>
    <slot></slot>
  `);const j=(e,{tag:i,is:o})=>{const d=globalThis.document?.createElement(i,{is:o}),c=d?S(d):[];return class g extends e{static Events=$;static template=A;static skipAttributes=[];static#s;static get observedAttributes(){return g.#l(),[...d?.constructor?.observedAttributes??[],"autopictureinpicture","disablepictureinpicture","disableremoteplayback","autoplay","controls","controlslist","crossorigin","loop","muted","playsinline","poster","preload","src"]}static#l(){if(this.#s)return;this.#s=!0;const t=new Set(this.observedAttributes);t.delete("muted");for(let s of c){if(s in this.prototype)continue;if(typeof d[s]=="function")this.prototype[s]=function(...r){this.#t();const a=()=>this.call?this.call(s,...r):this.nativeEl[s].apply(this.nativeEl,r);return this.loadComplete&&!this.isLoaded?this.loadComplete.then(a):a()};else{let r={get(){this.#t();let a=s.toLowerCase();if(t.has(a)){const p=this.getAttribute(a);return p===null?!1:p===""?!0:p}return this.get?.(s)??this.nativeEl?.[s]??this.#i[s]}};s!==s.toUpperCase()&&(r.set=async function(a){this.#t();let p=s.toLowerCase();if(t.has(p)){a===!0||a===!1||a==null?this.toggleAttribute(p,!!a):this.setAttribute(p,a);return}if(this.loadComplete&&!this.isLoaded&&await this.loadComplete,this.set){this.set(s,a);return}this.nativeEl[s]=a}),Object.defineProperty(this.prototype,s,r)}}}#o;#n;#r=!1;#e=!1;#a;#i;constructor(){super(),this.shadowRoot||(this.attachShadow({mode:"open"}),this.shadowRoot.append(this.constructor.template.content.cloneNode(!0))),this.load!==g.prototype.load&&(this.loadComplete=new O)}get loadComplete(){return this.#n}set loadComplete(t){this.#e=!1,this.#n=t,t?.then(()=>{this.#e=!0})}get isLoaded(){return this.#e}get nativeEl(){return this.#a??this.shadowRoot.querySelector(i)??this.querySelector(i)}set nativeEl(t){this.#a=t}get defaultMuted(){return this.hasAttribute("muted")}set defaultMuted(t){this.toggleAttribute("muted",!!t)}get src(){return this.getAttribute("src")}set src(t){this.setAttribute("src",`${t}`)}get preload(){return this.getAttribute("preload")??this.nativeEl?.preload}set preload(t){this.setAttribute("preload",`${t}`)}async#t(){if(this.#o)return;this.#o=!0,this.#u(),this.#c();for(let u of c)this.#d(u);const t=new Map,s=this.shadowRoot.querySelector("slot:not([name])");s?.addEventListener("slotchange",()=>{const u=new Map(t);s.assignedElements().filter(r=>["track","source"].includes(r.localName)).forEach(async r=>{u.delete(r);let a=t.get(r);a||(a=r.cloneNode(),t.set(r,a)),this.loadComplete&&!this.isLoaded&&await this.loadComplete,this.nativeEl.append?.(a)}),u.forEach(r=>r.remove())});for(let u of this.constructor.Events)this.shadowRoot.addEventListener?.(u,r=>{r.target===this.nativeEl&&this.dispatchEvent(new CustomEvent(r.type,{detail:r.detail}))},!0)}#d(t){if(Object.prototype.hasOwnProperty.call(this,t)){const s=this[t];delete this[t],this[t]=s}}#u(){const t=document.createElement(i,{is:o});t.muted=this.hasAttribute("muted");for(let{name:s,value:u}of this.attributes)t.setAttribute(s,u);this.#i={};for(let s of S(t))this.#i[s]=t[s];t.removeAttribute("src"),t.load()}async#c(){if(this.loadComplete&&!this.isLoaded&&await this.loadComplete,!this.nativeEl){const t=document.createElement(i,{is:o});t.part=i,this.shadowRoot.append(t)}this.nativeEl.muted=this.hasAttribute("muted")}attributeChangedCallback(t,s,u){this.#t(),t==="src"&&this.load!==g.prototype.load&&this.#h(),this.#p(t,s,u)}async#h(){this.#r&&(this.loadComplete=new O),this.#r=!0,await Promise.resolve(),await this.load(),this.loadComplete?.resolve(),await this.loadComplete}async#p(t,s,u){this.loadComplete&&!this.isLoaded&&await this.loadComplete,!["id","class",...this.constructor.skipAttributes].includes(t)&&(u===null?this.nativeEl.removeAttribute?.(t):this.nativeEl.setAttribute?.(t,u))}connectedCallback(){this.#t()}}};function S(e){let i=[];for(let o=Object.getPrototypeOf(e);o&&o!==HTMLElement.prototype;o=Object.getPrototypeOf(o))i.push(...Object.getOwnPropertyNames(o));return i}class O extends Promise{constructor(i=()=>{}){let o,d;super((c,f)=>{i(c,f),o=c,d=f}),this.resolve=o,this.reject=d}}const B=globalThis.document?j(HTMLElement,{tag:"video"}):class{};globalThis.document&&j(HTMLElement,{tag:"audio"});var H,R;const L=(H=globalThis.document)==null?void 0:H.createElement("template");L&&(L.innerHTML=`
  <div class="wistia_embed"></div>
  `);const M=(R=globalThis.document)==null?void 0:R.createElement("template");M&&(M.innerHTML=`
  <style>
    :host {
      display: inline-block;
      min-width: 300px;
      min-height: 150px;
      position: relative;
    }
    ::slotted(.wistia_embed) {
      position: absolute;
      width: 100%;
      height: 100%;
    }
  </style>
  <slot></slot>
  `);class q extends B{static template=M;static skipAttributes=["src"];get nativeEl(){var i;return((i=this.api)==null?void 0:i.elem())??this.querySelector("video")}async load(){var i;if((i=this.querySelector(".wistia_embed"))==null||i.remove(),!this.src)return;await new Promise(s=>setTimeout(s,50));const o=/(?:wistia\.com|wi\.st)\/(?:medias|embed)\/(.*)$/i,d=this.src.match(o)[1],c={autoPlay:this.autoplay,preload:this.preload??"metadata",playsinline:this.playsInline,endVideoBehavior:this.loop&&"loop",chromeless:!this.controls,playButton:this.controls,muted:this.defaultMuted};this.append(L.content.cloneNode(!0));const f=this.querySelector(".wistia_embed");f.id||(f.id=U(d)),f.classList.add(`wistia_async_${d}`),await D("https://fast.wistia.com/assets/external/E-v1.js","Wistia"),this.api=await new Promise(s=>{globalThis._wq.push({id:f.id,onReady:s,options:c})})}async attributeChangedCallback(i,o,d){if(i==="controls"){switch(await this.loadComplete,i){case"controls":this.api.bigPlayButtonEnabled(this.controls),this.controls?this.api.releaseChromeless():this.api.requestChromeless();break}return}super.attributeChangedCallback(i,o,d)}get duration(){var i;return(i=this.api)==null?void 0:i.duration()}play(){return this.api.play(),new Promise(i=>this.addEventListener("playing",i))}}const T={};async function D(e,i){return T[e]?T[e]:self[i]?self[i]:T[e]=new Promise((o,d)=>{const c=document.createElement("script");c.defer=!0,c.src=e,c.onload=()=>o(self[i]),c.onerror=d,document.head.append(c)})}let F=0;function U(e){const i=++F;return`${e}${i}`}globalThis.customElements&&!globalThis.customElements.get("wistia-video")&&globalThis.customElements.define("wistia-video",q);var z=q,V=new Set(["style","children","ref","key","suppressContentEditableWarning","suppressHydrationWarning","dangerouslySetInnerHTML"]),G={className:"class",htmlFor:"for"};function I(e){return e.toLowerCase()}function x(e){if(typeof e=="boolean")return e?"":void 0;if(typeof e!="function"&&!(typeof e=="object"&&e!==null))return e}function J({react:e,tagName:i,elementClass:o,events:d,displayName:c,defaultProps:f,toAttributeName:t=I,toAttributeValue:s=x}){const u=Number.parseInt(e.version)>=19,r=e.forwardRef((a,p)=>{var P,k;const m=e.useRef(null),E=e.useRef(new Map),C={},_={},b={},w={};for(const[n,l]of Object.entries(a)){if(V.has(n)){b[n]=l;continue}const h=t(G[n]??n);if(n in o.prototype&&!(n in(((P=globalThis.HTMLElement)==null?void 0:P.prototype)??{}))&&!((k=o.observedAttributes)!=null&&k.some(y=>y===h))){w[n]=l;continue}if(n.startsWith("on")){C[n]=l;continue}const v=s(l);if(h&&v!=null&&(_[h]=String(v),u||(b[h]=v)),h&&u){const y=x(l);v!==y?b[h]=v:b[h]=l}}if(typeof window<"u"){for(const n in C){const l=C[n],h=n.endsWith("Capture"),v=(d?.[n]??n.slice(2).toLowerCase()).slice(0,h?-7:void 0);e.useLayoutEffect(()=>{const y=m?.current;if(!(!y||typeof l!="function"))return y.addEventListener(v,l,h),()=>{y.removeEventListener(v,l,h)}},[m?.current,l])}e.useLayoutEffect(()=>{if(m.current===null)return;const n=new Map;for(const l in w)N(m.current,l,w[l]),E.current.delete(l),n.set(l,w[l]);for(const[l,h]of E.current)N(m.current,l,void 0);E.current=n})}if(typeof window>"u"&&o?.getTemplateHTML&&o?.shadowRootOptions){const{mode:n,delegatesFocus:l}=o.shadowRootOptions,h=e.createElement("template",{shadowrootmode:n,shadowrootdelegatesfocus:l,dangerouslySetInnerHTML:{__html:o.getTemplateHTML(_,a)}});b.children=[h,b.children]}return e.createElement(i,{...f,...b,ref:e.useCallback(n=>{m.current=n,typeof p=="function"?p(n):p!==null&&(p.current=n)},[p])})});return r.displayName=c??o.name,r}function N(e,i,o){var d;e[i]=o,o==null&&i in(((d=globalThis.HTMLElement)==null?void 0:d.prototype)??{})&&e.removeAttribute(i)}var Q=J({react:W,tagName:"wistia-video",elementClass:z,toAttributeName(e){return e==="muted"?"":e==="defaultMuted"?"muted":I(e)}});/*! Bundled license information:

ce-la-react/dist/ce-la-react.js:
  (**
   * @license
   * Copyright 2018 Google LLC
   * SPDX-License-Identifier: BSD-3-Clause
   *
   * Modified version of `@lit/react` for vanilla custom elements with support for SSR.
   *)
*/export{Q as default};
